@@ -1,6 +1,6 @@
 // VolumeScanner.cs
 // 
-// Copyright (C) 2008 - 2011 Patrick Ulbrich
+// Copyright (C) 2008 - 2012 Patrick Ulbrich
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ namespace Basenji.Gui
 		private ListStore				logStore;
 		private VolumeDatabase			database;
 		private IVolumeScanner			scanner;
-		private List<MetadataProvider>	mdps;
+		private MetadataProvider[]		mdps;
 		private StringBuilder			scannerLog;
 		private StatusUpdateTimer		timer;
 
@@ -58,28 +58,14 @@ namespace Basenji.Gui
 			//InitTreeView();
 			
 			mdps = null;
-			string extractorWarning = null;
 			
 			if (App.Settings.ScannerExtractMetaData && 
 			    (VolumeProber.ProbeVolume(drive) == VolumeProber.VolumeProbeResult.Filesystem)) {
 				
-				mdps = new List<MetadataProvider>();
-				
-/*				if (App.Settings.ScannerMetaDataProvider == 0) {
-					//mdps.Add(new TagLibMetadataProvider());
-				} else {*/
-					
-					string[] blacklist = App.Settings.ScannerExtractionBlacklist
-						.Split(new string[] { ", ", "," },
-						StringSplitOptions.RemoveEmptyEntries);
-					
-					try {
-						mdps.Add(new ExtractorMetadataProvider(blacklist));
-					} catch (DllNotFoundException) {
-						// libextractor package not installed
-						extractorWarning = S._("libExtractor not found. Metadata extraction disabled.");
-					}
-/*				}*/
+				mdps = new MetadataProvider[] {				
+					new TagLibMetadataProvider(),
+					new ArchiveMetadataProvider()
+				};
 			}
 			
 			// setup scanner options
@@ -144,10 +130,6 @@ namespace Basenji.Gui
 					                                          BoolToStr(App.Settings.ScannerExtractMetaData),
 					                                          BoolToStr(App.Settings.ScannerDiscardSymLinks),
 					                                          BoolToStr(App.Settings.ScannerComputeHashs)));
-						
-						if (extractorWarning != null)
-							UpdateLog(LogIcon.Warning, extractorWarning);
-					
 						break;
 					case VolumeType.AudioCdVolume:
 						UpdateLog(LogIcon.Info, string.Format(S._("Options: MusicBrainz enabled: {0}"),
@@ -186,24 +168,20 @@ namespace Basenji.Gui
 		}
 		
 		private void UpdateLog(LogIcon icon, string message) {
-			//string stockIcon;
 			Pixbuf pb;
 			string messageType;
 			string timeStr = DateTime.Now.ToShortTimeString();
 
 			switch(icon) {
 				case LogIcon.Info:
-					//stockIcon = Stock.DialogInfo;
 					pb = infoIcon;
 					messageType = "[ INFO  ]";
 					break;
 				case LogIcon.Warning:
-					//stockIcon = Stock.DialogWarning;
 					pb = warningIcon;
 					messageType = "[WARNING]";
 					break;
 				case LogIcon.Error:
-					//stockIcon = Stock.DialogError;
 					pb = errorIcon;
 					messageType = "[ ERROR ]";
 					break;
@@ -211,7 +189,6 @@ namespace Basenji.Gui
 					throw new Exception("Invalid LogIcon");
 			}
 
-			//Pixbuf pb = this.RenderIcon(stockIcon, IconSize.Menu, string.Empty);
 			logStore.AppendValues(pb, timeStr, message);
 
 			scannerLog.AppendFormat("[{0}] {1} {2}\n", timeStr, messageType, message);
@@ -284,7 +261,7 @@ namespace Basenji.Gui
 			/* remove event handlers from the extern VolumeDatabase object */
 			database.BeginWriteAccess	-= database_BeginWriteAccess;
 			database.EndWriteAccess		-= database_EndWriteAccess;
-
+			
 			//// remove timeout handler (installed in ctor)
 			//// TODO : comment from GUI_PUSH.txt
 			//m_timer.Remove();
@@ -317,15 +294,15 @@ namespace Basenji.Gui
 		
 		#region VolumeDatabase event handlers (executed on the scanner thread ?)
 		// TODO : 
-		// what is the executing thread of those MediaDB events rised on the MediaDB by the MediaScanner?
+		// what is the executing thread of those VolumeDB events rised on the VolumeDB by the VolumeScanner?
 		// in case of a different thread, 
-		// is the MediaScanner required to raise the event on the the tread of the MediaDB?
+		// is the VolumeScanner required to raise the event on the the tread of the VolumeDB?
 		private void database_BeginWriteAccess(object sender, EventArgs e) {
-			timer.LedState = true; // LED on
+			timer.RequestLedUpdate(true); // LED on
 		}
-
+		
 		private void database_EndWriteAccess(object sender, EventArgs e) {
-			timer.LedState = false; // LED off
+			timer.RequestLedUpdate(false); // LED off
 		}
 		#endregion
 		
@@ -396,20 +373,18 @@ namespace Basenji.Gui
 		 */
 		private class StatusUpdateTimer
 		{
+			/* GUI update frequency: (1000 / TIMEOUT_INTERVAL) Hz */
 			private const uint TIMEOUT_INTERVAL = 20;
 
 			private VolumeScanner	vscanner;
 			private volatile bool	remove;
-
-			//private Led			  led;
-			private volatile bool	ledState;
-
+			private volatile bool	ledOnRequest;
+			private volatile bool	ledOffRequest;
+			private long			lastLedUpdate;
+			
 			public StatusUpdateTimer(VolumeScanner vs) {
 				this.vscanner	= vs;
 				this.remove		= true;
-
-				//this.led		  = new Led(vscanner.imgLed, false);
-				//m_ledState  = false;
 			}
 
 			public void Install() {
@@ -418,19 +393,43 @@ namespace Basenji.Gui
 
 				remove					= false;
 				vscanner.led.LedState	= false;
-				ledState				= false;
+				ledOnRequest			= false;
+				ledOffRequest			= false;
 
 				GLib.Timeout.Add(TIMEOUT_INTERVAL, delegate {
+					/* get local copies of thread shared vars */
 					bool persist = !remove;
-
-					/* update counter labels */
+					bool ledOnReq = ledOnRequest;
+					bool ledOffReq = ledOffRequest;
+					
+					/* 
+					 * update counter labels 
+					 */
 					if (vscanner.scanner != null)
 						vscanner.volEditor.UpdateInfo(vscanner.scanner.VolumeInfo);
 
-					/* LED (database access indicator) */
-					if (ledState != vscanner.led.LedState)
-						vscanner.led.LedState = ledState; /* toggle LED state */
-
+					/* 
+					 * update database write access indicator LED
+					 */
+					if (!persist) {
+						/* switch off the LED when the timer is going to be removed */
+						vscanner.led.LedState = false;
+					} else {
+						if (ledOnReq) {
+							if (!vscanner.led.LedState)
+								vscanner.led.LedState = true; /* toggle LED state */
+							
+							ledOnRequest = false;
+							lastLedUpdate = DateTime.Now.Ticks;
+						}
+						
+						/* switch off the LED if it hasn't been switched on during the last ~300 ms.
+						 * (i.e. make the LED glow for at least 300 ms) */
+						if ( vscanner.led.LedState && ledOffReq &&
+							(((DateTime.Now.Ticks - lastLedUpdate) / 10000) >= 300) )
+							vscanner.led.LedState = false;
+					}
+					
 					return persist;
 				});
 			}
@@ -439,9 +438,20 @@ namespace Basenji.Gui
 				remove = true;
 			}
 
-			public bool LedState {
-				get { return ledState; }
-				set { ledState = value; }
+			public void RequestLedUpdate(bool on) {
+				if (on) {
+					/* remove possibly pending previous switch-off request */
+					ledOffRequest = false;
+					
+					/* request LED switch-on */
+					ledOnRequest = true;
+					
+				} else {
+					/* don't disable the ledOnRequest here !*/
+					
+					/* request LED switch-off */
+					ledOffRequest = true;
+				}
 			}
 		}
 
@@ -466,7 +476,7 @@ namespace Basenji.Gui
 			base.BuildGui();
 			
 			// general window settings
-			SetDialogStyle();
+			SetModal();
 			//this.DefaultWidth		= 580;
 			this.DefaultHeight		= 600;
 			this.Title				= S._("VolumeScanner");
